/***************************************************************************
                          cdctag.cpp  -  description
                             -------------------
    begin                : Sun Aug 24 2003
    copyright            : (C) 2003 by Daniel Muller
    email                : dan@verliba.cz
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "cdctag.h"

#include <string>
#include "cpcre.h"
#include <iostream>

using std::string;
using namespace std;
using nUtils::cPCRE;

namespace nDirectConnect
{

cDCTagParser cDCTag::mParser;

cDCTagParser::cDCTagParser()
{
	/**
		the order of subpatterns
		0 - the whole tag
		1 - ++ or DCGUI
		2 - the version number
		3 - mode
		4 - the rest (call it body)

	*/
	if(!mTagRE.Compile("<(\\+\\+|DCGUI) ?V\\:([^,]+),M:(A|P)([^>]*)>"))
		throw "Error in tag general REGEX";
	/**
		0 - whole Hubs part
		1 - norm user hubs
		2 - reg hubs
		3 - op hubs
	*/
	if(!mHubsRE.Compile("H\\:(\\d+)(\\/\\d+)?(\\/\\d+)?"))
		throw "Error in hubs general REGEX";
	if(!mSlotsRE.Compile("S\\:(\\d+)"))
		throw "Error in slots general REGEX";
}

cDCTag::cDCTag(const std::string &desc, cDCConf &config) : mC(config)
{
	string str;

	mClientType = eCT_NOTAG;
	mClientMode = eCM_NOTAG;
	mPositionInDesc = -1;

	if(mParser.mTagRE.Exec(desc) >= 0)
	{
		// copy tag's parts
		mPositionInDesc = mParser.mTagRE.StartOf(0);
		mParser.mTagRE.Extract(0,desc,mTag);
		mParser.mTagRE.Extract(4,desc,mTagBody);

		// determine client's type
		mParser.mTagRE.Extract(1,desc,str);
		if( str == "++" ) mClientType = eCT_PLUSPLUS;
		else if (str == "DCGUI") mClientType = eCT_DCGUI;
		else if (str.size() ) mClientType = eCT_UNKNOWN;

		// determine client's mode
		mParser.mTagRE.Extract(3,desc,str);
		if( str == "A" ) mClientMode = eCM_ACTIVE;
		else if (str == "P") mClientMode = eCM_PASSIVE;

	}
}

cDCTag::~cDCTag(){
}

};


/*!
    \fn nDirectConnect::cDCTag::ValidateTag(ostream &os)
 */
bool nDirectConnect::cDCTag::ValidateTag(ostream &os, int conn_type)
{
	istringstream *is;
	string str;
	int hub=-1,slot=-1;

	if(mPositionInDesc == -1)
	{
		return true;
	}

	if(mParser.mHubsRE.Exec( mTagBody ) >= 2)
	{
		mParser.mHubsRE.Extract(1,mTagBody,str);
		is = new istringstream(str);
		*is >> hub;
		delete is;
	}

	if(mParser.mSlotsRE.Exec( mTagBody ) >= 2)
	{
		mParser.mSlotsRE.Extract(1,mTagBody,str);
		is = new istringstream(str);
		//is.clear();
		//is.seeg(0,istream::beg);
		//is.str(str);
		*is >> slot;
		delete is;
	}

	if((hub < 0) || (slot < 0))
	{
		os << "Can't parse your DC++ tag, I suppose it's wrong";
		return false;
	}
	if(hub > mC.tag_max_hubs)
	{
		os << "Too many open hubs, max is:" << mC.tag_max_hubs;
		return false;
	}
	if(slot > mC.tag_max_slots[conn_type])
	{
		os << "Too many open slots for your connection type, max is:" << mC.tag_max_slots[conn_type];
		return false;
	}
	if(slot < mC.tag_min_slots[conn_type])
	{
		os << "Too little open slots for your connection type, min is:" << mC.tag_min_slots[conn_type];
		return false;
	}
	if( (mC.tag_max_hs_ratio * slot) < hub )
	{
		os << "Too little slots for the number of hubs (hubs/slots ratio), max ratio is:" << mC.tag_max_hs_ratio;
		return false;
	}

	return true;
}
