/***************************************************************************
 *   Copyright (C) 2003 by Dan Muller                                      *
 *   dan@verliba.cz                                                        *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 ***************************************************************************/
#include "cusercollection.h"
#include "cuser.h"

namespace nDirectConnect {

void cUserCollection::ufDoNickList::AppendNickList(crope &List, cUser *User)
{
	List.append(User->mNick.data(),User->mNick.size());
	List.append(mSep.data(),mSep.size());
}

cUserCollection::cUserCollection(bool KeepNickList) :
	mNickListMaker(mNickList),
	mKeepNickList(KeepNickList),
	mRemakeNextNickList(true)
{
	SetClassName("cUsrColl");
}


cUserCollection::~cUserCollection()
{
}

void cUserCollection::Nick2Key(const std::string &Nick, std::string &Key)
{
	Key.assign(Nick);
	std::transform(Key.begin(), Key.end(), Key.begin(), ::tolower);
}

void cUserCollection::Nick2Hash(const std::string &Nick, size_t &Hash)
{
	string Key;
	Nick2Key(Nick,Key);
	Hash = mHasher(Key);
}

////////////////////////// CONTAINS

bool cUserCollection::ContainsHash(const size_t &Hash)
{
	return mUserHash.find(Hash) != mUserHash.end();
}


/////////////////////////// GET USER BY..

cUser * cUserCollection::GetUserByHash(const size_t &Hash)
{
	tUHIt uhit = mUserHash.find(Hash);
	if( uhit !=  mUserHash.end()) return *(uhit->second);
	return NULL;
}

///////////////////////////// ADD USER

bool cUserCollection::AddWithHash(cUser *User, const size_t &Hash)
{
	if(ContainsHash(Hash)) { if(Log(0))LogStream() << "Trying to add " << Hash << " twice" << endl; return false; }

	iterator ulit = mUserList.insert(mUserList.begin(), User);
	if(ulit == mUserList.end()) { if(Log(0))LogStream() << "Can't add " << Hash << " into the list" << endl; return false; 	}

	pair<tUHIt,bool> P = mUserHash.insert(tNickUserITPair(Hash,ulit));
	if ( P.second ) {
		if( mKeepNickList && ! mRemakeNextNickList ) mNickListMaker(User);
		if(Log(3))LogStream() << "Successfully added " << Hash << endl;
	}
	else {
		if(Log(0))LogStream() << "Can't add " << Hash << endl;
		mUserList.erase(ulit);
		return false;
	}

	// debug check
	if(!ContainsHash(Hash)) { if(Log(0))LogStream() << "ERROR adding " << Hash << endl; return false; }

	return true;
}

bool   cUserCollection::Add        (cUser *User                    )
{ if(User) return AddWithHash(User, Nick2Hash(User->mNick)); else return false; }

///////////////////////////////// REMOVE USER

bool cUserCollection::RemoveByHash(const size_t &Hash)
{
	tUHIt uhit = mUserHash.find(Hash);
	if( uhit !=  mUserHash.end())
	{
		// if(Log(0))LogStream() << (*(uhit->second))->mNick << " to remove :" << Hash << endl;
		mUserList.erase( uhit->second );
		mUserHash.erase( uhit );
		// remake nicklist
		mRemakeNextNickList = mKeepNickList;
		if(Log(3))LogStream() << "Removed " << Hash << " successfully from " << mNickListMaker.mStart << endl;
		return true;
	}
	if(Log(3))LogStream() << "Removing User that doesn't exist :" << Hash << endl;
	return false;
}

bool   cUserCollection::Remove      (cUser *User       )
{ if(User) return RemoveByHash(Nick2Hash(User->mNick)); else return false; }

////////////////////////////////// the rest

crope &cUserCollection::GetNickList()
{
	if(mKeepNickList && mRemakeNextNickList)
	{
		if(Log(4)) LogStream() << "Remaking " << mNickListMaker.mStart << " of " << mUserHash.size() << endl;
		if(Log(4)) CoutAllKeys();
		mNickListMaker.Clear();
		for_each(begin(),end(),mNickListMaker);
		mRemakeNextNickList = false;
	}
	return mNickList;
}


void cUserCollection::SendToAll(crope &Data, bool UseCache, bool AddPipe)
{
	if(AddPipe) Data.append('|');
	if (UseCache)
		mSendAllCache.append(Data);
	else{
		if(Log(4))CoutAllKeys();
		if(Log(4))LogStream() << "SendAll BEGIN" << endl;
		for_each(this->begin(),this->end(),ufSend<crope>(Data));
		if(Log(4))LogStream() << "SendAll END" << endl;
	}
	if(AddPipe) Data.erase(Data.size()-1,1);
}

void cUserCollection::SendToAll(string &Data, bool UseCache, bool AddPipe)
{
	if(AddPipe) Data.append("|");
	if (UseCache)
		mSendAllCache.append(Data.data(),Data.size());
	else{
		if(Log(4))CoutAllKeys();
		if(Log(4))LogStream() << "SendAll BEGIN" << endl;
		for_each(this->begin(),this->end(),ufSend<string>(Data));
		if(Log(4))LogStream() << "SendAll END" << endl;
	}
	if(AddPipe) Data.erase(Data.size()-1,1);
}

void cUserCollection::SendToAllWithNick(string &Start, string &End)
{
	for_each(this->begin(),this->end(),ufSendWithNick<std::string>(Start,End));
}

void cUserCollection::FlushCache()
{
	if(mSendAllCache.size())
	{
		SendToAll(mSendAllCache, false);
		mSendAllCache.erase(0,mSendAllCache.size());
	}
}

int cUserCollection::StrLog(ostream & ostr, int level)
{
	if(cObj::StrLog(ostr,level))
	{
		LogStream()   << "(" << mNickListMaker.mStart ;
		LogStream()   << ") "<< "[ " << mUserHash.size() << "/" << mUserList.size() << " ] ";
		return 1;
	}
	return 0;
}

void cUserCollection::CoutAllKeys()
{
	tUHIt i = mUserHash.begin();
	int j = 0;
	cout << mNickListMaker.mStart << " ";
	for(;i!=mUserHash.end() && j < mUserHash.size()*2;++i, ++j)
	{

		cout << i->first << " " << (*(i->second))->mNick << " (" << ContainsHash(i->first) <<")  " ;
	}
	cout << endl;
}

};
