/***************************************************************************
 *   Copyright (C) 2003 by Dan Muller                                      *
 *   dan@verliba.cz                                                        *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 ***************************************************************************/
#ifndef NDIRECTCONNECTCUSERCOLLECTION_H
#define NDIRECTCONNECTCUSERCOLLECTION_H

//#include <ext/slist>
#include <list>
#include <ext/rope>
#include <string>
#include "tstringhashmap.h"
#include <map>
#include "cobj.h"

//using __gnu_cxx::slist;
using std::list;
using __gnu_cxx::rope;
using std::string;
using namespace nUtils;

namespace nDirectConnect {

class cUser;

/**
a structure that allows to insert and remove users, to quilclky iterate and holda commun sendall buffer, provides also number of sendall functions

supports: quick iterating with (restrained) constant time increment (see std::slist) , logaritmic finding, adding, removing, testing for existence.... (see std::hash_map)

@author Daniel Muller
*/

class cUserCollection :  public cObj{
public:
	// single linked list is good for interating over all
	typedef list<cUser*> tUserSList;
	// invariant iterator ... good for insertion etc..
	typedef tUserSList::iterator iterator;

	// hash of key (lowercase nick) -> mapped to tULIt .. one can get cUserDC* from it
//	typedef tStringHashMap<iterator> tUserHash;
	typedef map<size_t,iterator> tUserHash;
	typedef tUserHash::iterator tUHIt;

//	typedef pair<string,iterator> tNickUserITPair;
	typedef pair<size_t,iterator> tNickUserITPair;

	template <class DATA_T> struct ufSend : public unary_function<void, iterator>
	{
		//Data to send
		DATA_T &mData;

		ufSend(DATA_T &Data):mData(Data){};
		void operator() (cUser *User){ if(User && User->mInList && User->mxConn && User->mxConn->ok) User->mxConn->Send(mData, false); }
	};

	template <class DATA_T> struct ufSendWithNick : public unary_function<void, iterator>
	{
		//Data to send
		DATA_T &mDataStart, &mDataEnd;

		ufSendWithNick(DATA_T &DataS,DATA_T &DataE):mDataStart(DataS), mDataEnd(DataE){};
		void operator() (cUser *User)
		{
			if(User && User->mxConn && User->mxConn->ok && User->mInList )
			{
				User->mxConn->Send(mDataStart, false);
				User->mxConn->Send(User->mNick, false);
				User->mxConn->Send(mDataEnd, true);
			}
		}
	};

	struct ufDoNickList : public unary_function<void, cUserCollection::iterator>
	{
		ufDoNickList(crope &List):mList(List){}
		void Clear() {
			mList.erase(0,mList.size());
			mList.append(mStart.data(), mStart.size());
		}
		void operator() (cUser *usr){ AppendNickList(mList, usr);};
		void AppendNickList(crope &List, cUser *User);

		string mStart;
		string mSep;

		crope &mList;
	};

private:
	tUserSList mUserList;
	tUserHash  mUserHash;
	crope mSendAllCache;
	crope mNickList;
	ufDoNickList mNickListMaker;
	bool mKeepNickList;
	bool mRemakeNextNickList;
	hHashStr mHasher;

public:
	cUserCollection(bool KeepNickList=false);
	~cUserCollection();

	iterator begin(){ return mUserList.begin();}
	iterator end()  { return mUserList.end();  }

	virtual int StrLog(ostream & ostr, int level);
	void SetNickListStart(const string &Start){mNickListMaker.mStart = Start;}
	void SetNickListSeparator(const string &Separator){mNickListMaker.mSep = Separator;}
	crope &GetNickList();
	const char *GetNickListChar() {return GetNickList().c_str();}

	void   Nick2Key (const std::string &Nick, std::string &Key);
	void   Nick2Hash(const std::string &Nick, size_t &Hash);
	size_t Nick2Hash(const std::string &Nick){string Key; Nick2Key(Nick,Key); return mHasher(Key); }

	cUser* operator [](const string &Nick){return GetUserByNick(Nick);}

	cUser* GetUserByHash(const size_t &Hash);
	cUser* GetUserByKey (const string &Key ){ return GetUserByHash(mHasher(Key));}
	cUser* GetUserByNick(const string &Nick){ return GetUserByHash(Nick2Hash(Nick));}

	bool   ContainsHash(const size_t &Hash);
	bool   ContainsKey (const string &Key ){ return ContainsHash(mHasher(Key)); }
	bool   ContainsNick(const string &Nick){ return ContainsHash(Nick2Hash(Nick)); }

	bool   AddWithHash(cUser *User, const size_t &Hash);
	bool   AddWithKey (cUser *User, const string &Key ){ return AddWithHash(User, mHasher(Key));}
	bool   AddWithNick(cUser *User, const string &Nick){ return AddWithHash(User, Nick2Hash(Nick)); }
	bool   Add        (cUser *User                    );

	bool   RemoveByHash(const size_t &Hash);
	bool   RemoveByKey (const string &Key ){ return RemoveByHash(mHasher(Key)); }
	bool   RemoveByNick(const string &Nick){ return RemoveByHash(Nick2Hash(Nick));}
	bool   Remove      (cUser *User       );

	void   SendToAllWithNick(string &Start, string &End);
	void   SendToAll(string &Data, bool UseCache = false, bool AddPipe=true);
	void   SendToAll(crope  &Data, bool UseCache = false, bool AddPipe=true);
	void   FlushCache();
	void   CoutAllKeys();
};

};

#endif
